#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> memsas.tcl : tcl routine that sets up a window     <--#
#-->                   that displays the properties of all   <--#
#-->                   of the memsas daemons that are        <--#
#-->                   running on the current machine.       <--#
#-->                                                         <--#
#--> D.Appelbe Daresbury NPG 01/05/04                        <--#
#    V Pucknell 2005=>
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#

set m [package require Tclx]     ;# for kill 
if {$m != 8.4} then {midas-warning "problem with package Tclx: $m\nEnsure that the software is correctly installed"}

proc memsas_ctrl {} {
    #--> The first thing to do is to gather all the required 
    #--> information.
    memsas_ctrl.get_data
    #--> Now we shall generate the GUI used to display the
    #--> info
    memsas_ctrl.make_frame
    #--> Now configure our tags
    memsas_ctrl.configure_tags
    #--> Now paint our data to the appropriate places
    memsas_ctrl.paint_alloc
    memsas_ctrl.paint_daemons
}

#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> Subroutines called by memsas_ctrl                       <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc memsas_ctrl.get_data {} {
    global env slmemalloc
    global sldaemons

    set slmemalloc     [ list "Name           Port     Capability" ]
    set name "None Running"
    set port 0
    set cap  "0x00000000000000"

    #--> Read our current resources
    set resources {}
    set resources [ :experiment resources ]
    #--> Now extract the required data
    foreach res $resources {
    foreach r $res {
        switch [ first $r ] {
        name        { set name [ second $r ] }
        port        { set port [ second $r ] }
        capability  { set cap  [ second $r ] }
        }
    }
    if { $name == "SortSas" } then {
        set tmptxt [ format "%-15s%-9s%-17s" $name $port $cap ]
        lappend slmemalloc $tmptxt
    }
    }

    #--> Now get data about memsas daemons that are currently running
    set sldaemons      [ list "Port      PID       Owner" ]  
    set filelist       [ :directory files :path "$env(MIDASTempDir)" ]
    set portlist       {}
    set pidlist        {}
    set present        0
    set actr           0
    set bctr           0
    foreach f $filelist {
    set fname [ first $f ]
    set present 0
    set present [ string match "memsas.port.*" $fname ]
    if { $present == 1 } then {
        lappend portlist $fname
        set actr [ expr $actr + 1 ]
    }
    set present 0
    set present [ string match "memsas.pid.*" $fname ]
    if { $present == 1 } then {
        lappend pidlist $fname
        set bctr [ expr $bctr + 1 ]
    }
    }
    #--> do we have parity between the two ?
    if { $actr == $bctr } then {
    foreach port $portlist {
        set z  [ catch { open [ file join "$env(MIDASTempDir)" $port ] r } f ]
        if { $z == 0 } then {
        gets $f aa
        close $f
        foreach pid $pidlist {
            set bb [ third [ split $pid "." ] ]
            if { $aa == $bb } then {
            set myport [ third [ split $port "." ] ]
            set owner  [ file attributes [ file join "$env(MIDASTempDir)" $port ] -owner ]
            lappend sldaemons [ format "%-10s%-10s%-10s" $myport $aa $owner ]
            }
        }
        } else {
        midas-warning "Could not open file : $port"
        }
    }
    } elseif { $actr > $bctr } then {
    } else {
    }
   
}

proc memsas_ctrl.make_frame {} {
    global slmemalloc
    global sldaemons

    :frame FMEMSAS :label "MIDASsort: Memsas Information/Control Frame" :resize F :transient T

    :panel PMEMSAS :layout V :padx 5 :pady 5
    :layout H
    :message MMEMSAS1 :label "Allocated Resources" :colour "-fg black" :bold T
    :next-row T
    :galley GMEMSAS1 :cols 50 :rows 4 :deaf T :font variable :see 0 :scroll N
    :next-row T
    :message MMEMSAS2 :label "The following Memsas Daemons are running" :colour "-fg black" :bold T
    :next-row T
    :panel PMEMSAS1 :layout L :padx 5 :pady 5
    :layout H
    :list LMEMSAS1 :cols 35 :rows 7  :see 0 :scroll V :command memsas_ctrl.select
    :panel PMEMSAS2 :layout L :padx 5 :pady 5
    :layout V
    :button BMEMSAS1 :label "Kill Daemon" :command memsas_ctrl.kill_daemon
    :next-row T
    :next-row T
    :button BMEMSAS3 :label "Refresh" :command memsas_ctrl.refresh
    :next-row T
    :next-row T
    :button BMEMSAS2 :label "Cancel" :command memsas_ctrl.leave
    :show T
}



proc memsas_ctrl.paint_alloc {} {
    global slmemalloc

    :modify GMEMSAS1 :empty T
    set lctr 0
    foreach line $slmemalloc {
    if { $lctr == 0 } then {
        :modify GMEMSAS1 :usingtag tagdisp1 :strings "$line\n" :see 0
    } else {
        :modify GMEMSAS1 :usingtag tagdisp2 :strings "$line\n" :see 0
    }
    set lctr [ expr $lctr + 1 ]
    if { $lctr >= 2 } then {
        set lctr 0
    }
    }
}

proc memsas_ctrl.paint_daemons {} {
    global sldaemons

    :modify LMEMSAS1 :empty T :strings $sldaemons :see 0
}

proc memsas_ctrl.leave {} {
    destroy-frame FMEMSAS 
}

proc memsas_ctrl.select {w v i} {
    global mc_port mc_pid mc_listval

    set mc_listval $i
    set mc_port [first $v]
    set mc_pid  [second $v]

}

proc memsas_ctrl.kill_daemon {} {
    global mc_port mc_pid mc_listval

    if { $mc_listval > 0 } then {
    set z [ catch { kill $mc_pid } m]
    if { $z == 0 } then {
        memsas_ctrl.get_data
        memsas_ctrl.paint_daemons
    } else {
        midas-warning "Could not kill MemSas daemon: $m"
    }
    } else {
    midas-warning "This is not a valid MemSas daemon"
    }
}

proc memsas_ctrl.refresh {} {
    memsas_ctrl.get_data
    memsas_ctrl.paint_alloc
    memsas_ctrl.paint_daemons
}

proc memsas_ctrl.configure_tags {} {
    :modify GMEMSAS1 :createtag "tagdisp1 -foreground red"
    :modify GMEMSAS1 :createtag "tagdisp2 -foreground blue"
}

#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#-->                                                         <--#
#--> Tcl file to kill the memsas server, and associated segs <--#
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc kill_memsas {} {
    global env EGallow

    #--> First get the port of the sas we are using
    set port      0
    set pid       ""
    set resources {}
    set name      ""
    set resources [ :experiment resources ]
    foreach res $resources {
    foreach r $res {
        switch [ first $r ] {
        name        { set name [ second $r ] }
        host        { set host [ second $r ] }
        role        { set role [ second $r ] }
        type        { set type [ second $r ] }
        port        { set port [ second $r ] }
        }
    }
    if { $name == "SortSas" } then {
        set myfile [file join $env(MIDASTempDir) memsas.port.$port]
        set z [ catch { open $myfile r } f ]
        if { $z == 0 } then {
        gets $f pid
        close $f
        set zz [ catch { kill $pid } m ]
        }
        midas-check-for-temp
        set z [ catch { open "SortSas" w } f ] 
        if { $z == 0 } then {
        puts $f "((name $name)"
        puts $f " (host $host)"
        puts $f " (role $role)"
        puts $f " (type $type)"
        puts $f " (ipaddress 123456)"
        puts $f " (port $port))"
        close $f
        
        set EGallow 0x0
        #--> Do it this way to avoid any errors
        set za [ catch { eg "(apparatus :name 'SortSas' !free)" } m ]
        :apparatus detach :name "SortSas"
        
        }
    }
    }

}

#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> check_resource_settings.tcl                             <--#
#-->                                                         <--#
#--> Routine to check to see that you have a spectrum server <--#
#--> running and that you have the correct resources         <--#
#--> claimed, if these are not then it will launch/set them  <--#
#--> for you .........                                       <--#
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc check_server {} {
    global mydebug
    global env
    global SortSasport
    global serverprog

    global scldbg
#   set scldbg 1
    set scldbg 0

    set filelist     [ :directory files :path "$env(MIDASTempDir)" ]
    set pidlist      {}
    set pidlistrun   {}
    set portlist     {}
    set status       0
    set serverprog   "memsas"
    set serverport   -1
    set SortSasport 0
    set actr         0
    set bctr         0
    set sasown       0
    set sasrun       0
    set sasownctr    0
    set sasownlist   {}
    
    if { $env(platform) == "windows" } then {check_win_server; return}

    #--> First we need to look for the presence of any memsas
    #--> files in temporary file region
    foreach f $filelist {
       set present 0
       set fname [ first $f ]
       set present [ string match "memsas.pid.*" $fname ]
       if { $present == 1 } then {
           lappend pidlist $fname
           set actr [ expr $actr +1 ]
       }
       set present 0
       set present [ string match "memsas.port.*" $fname ]
       if { $present == 1 } then {
           lappend portlist $fname
           set bctr [ expr $bctr +1 ]
       }
    }

    #--> Now check for parity between what we believe is running 
    #--> and the true state of affairs.
    foreach port $portlist {
        set z [ catch { open [ file join "$env(MIDASTempDir)" $port ] r } f ]
        if { $z == 0 } then {
           gets $f aa
           close $f
        #--> Check to see if this process is running
           set zz [ catch { kill USR1 $aa } m ]
           if { $zz != 0 } then {
               set sasown [ file owned [ file join "$env(MIDASTempDir)" $port ] ]
               if { $sasown == 1 } then {
            #--> The process does not exist so remove the locks
                  set zzz [ catch { file delete [ file join "$env(MIDASTempDir)" $port ] } m]
                  if { $zzz != 0 } then {
                     midas-warning "Could not remove lockfile : [file join $env(MIDASTempDir) $port]"
                  }
                  set zzz [ catch { file delete [ file join "$env(MIDASTempDir)" "memsas.pid.$aa" ] } m]
                  if { $zzz != 0 } then {
                     midas-warning "Could not remove lockfile : [file join $env(MIDASTempDir) memsas.pid.$aa]"
                  }
               } else {
                set sasrun 1
               }
           } else {
            #--> Ok so we are running but do we own a memsas?
               set sasown 0
               set sasown [ file owned [ file join "$env(MIDASTempDir)" $port ] ]
               if { $sasown == 1 } then {
                  set sasownctr [ expr $sasownctr +1 ]
                  lappend sasownlist $port
               } else {

               }
            #--> Just set a flag to say that there is a running sas
               set sasrun 1
            }
        }
    }

    #--> As we have parity, now check what resources are allocated
    read_resource_settings
    
    #--> Is the SortSas allocated ..?
    if { $SortSasport > 0 } then {
    #--> is there a server running ?
       if { $sasrun == 1 } then {
        #--> do we own a running port ?
          if { $sasownctr > 0 } then {
        #--> check that what we own and what is runing are the same
             set ohmy [ lsearch $sasownlist memsas.port.$SortSasport ]
             if { $ohmy == -1 } then {
            #--> The running port is not the same as the resource so start
                set status 3
             } else {
            #--> All is well with the world
                set status 4
             }
          } else {
        #--> In this case deallocate and then create
            set status 2
          }
       } else {
          set status 1
       }
    } else {
    #--> no local sas allocated, in resource file
       set status 0
    }
    #--> Now Determine what to do ...
    set mystatus 0
    set myctr    0
    while { $status < 4 } {
       set mystatus $status
       if { $mystatus == 0 } then {
        #--> No SortSas resource allocated
          set status [ create_port -1 ]
       } elseif { $mystatus == 1 } then {
        #--> resource, not running
          set status [ create_port [ expr $SortSasport - 10230 ] ]
       } elseif { $mystatus == 2 } then {
        #--> resource, run, not owner
          set status [ create_port -1 ]
       } elseif { $mystatus == 3 } then {
        #--> Resource, run, own, dif between runport and resource
        set status [ reset_resource $SortSasport]
       } else {
        #--> All Ok
          set status 4
       }
       if { $myctr < 10 } then {
           set myctr [ expr $myctr + 1 ]
       } else {
           break
       }
    } 

    set env(MIDAS_MEMSAS_PORT) $SortSasport
}

proc check_win_server {} {
    global SortSasport

    read_resource_settings               ;    #check what resources are allocated
    
    #--> Is the SortSas allocated ..?
    if { $SortSasport > 0 } then {
        set status 4                     ;   # a useable running sas 
    } else {
    #--> no local sas allocated, in resource file
       set SortSasport 10230
       set status [reset_resource $SortSasport]
    }
    
    if {$status != 4} then {midas-warning "SortSas does not appear to have the correct status"}
    
    set env(MIDAS_MEMSAS_PORT) $SortSasport
}

#--> ======================================================= <--#
proc read_resource_settings {} {
    global env
    global SortSasport
    global resources

    #--> Ok now lets do the difficult stuff .. check the resources
    set resources {}
    set resources [ :experiment resources ]
    #--> Now look for our killer app ...  SortSas
    set present    0
    set name       ""
    set capability ""
    set port       ""
    set host       ""
    set role       ""
    set type       ""
    set ip         ""

    set SortSasport 0

    foreach res $resources {
       foreach r $res {
          switch [ first $r ] {
            name          { set name [ second $r ] }
            host          { set host [ second $r ] }
            role          { set role [ second $r ] }
            type          { set type [ second $r ] }
            port          { set port [ second $r ] }
            capability    { set cap  [ second $r ] }
          }
       }

       if { $name == "SortSas" } then {
           set present 1
           set SortSasport $port
           if { $capability != "0x0000000000000000" } then {set present 2}
           break;
       }
    }
}

#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc readpipe arg {
    set data ""
    set fileid [ open |$arg r ]
    set data   [ read $fileid ]
    close      $fileid
    return $data
}
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc create_port arg {
    global env SortSasport
    global serverprog scldbg
    
    if { $arg == -1 } then {
        if {$scldbg} then {
          set pid [ exec $serverprog.dbg -d-1 -c -p-1 & ]
        } else {
          set pid [ exec $serverprog -c -p-1 & ]
    #--> Have to add 1 to pid as the daemon forks ....
          incr pid
        }
    #--> Give the spectrum server a chance to search if need be ..
        for {set i 0} {$i < 10} {incr i} {
            set z [ catch { open "[file join $env(MIDASTempDir) memsas.pid.$pid]" r} f]
            if {$z == 0} {break}
            after 1000
        }
        if { $z != 0 } then {
            midas-warning "Could not find [file join $env(MIDASTempDir) memsas.pid.$pid] $f"
            return 16
        }
    #--> Now read the SortSas port from this file
        gets $f SortSasport
        close $f
        set rc 3
    } else {
    #--> The SortSasport is specified
        if {$scldbg} then {
          set pid [ exec $serverprog.dbg -d-1 -c -p$arg & ]
        } else {
          set pid [ exec $serverprog -c -p$arg & ]
    #--> Have to add 1 to pid as the daemon forks ....
          incr pid
        }
        set rc 4
    }
}
#--> ======================================================= <--#
proc reset_resource arg {
    global env EGallow EGRetries
    global resources mydebug SortSasport
    global afterstring
    #--> This procedure is used to reallocate resources
    #--> The first thing to do is to remove the current resource
    set directory [file join $env(MIDASTempDir) tcl[pid]]
    set name       ""
    set capability ""
    set port       ""
    set host       ""
    set role       ""
    set type       ""
    set ip         ""
    set status     0

    foreach res $resources {
       foreach r $res {
           switch [ first $r ] {
              name          { set name [ second $r ] }
              host          { set host [ second $r ] }
              role          { set role [ second $r ] }
              type          { set type [ second $r ] }
              port          { set port [ second $r ] }
             capability    { set cap  [ second $r ] }
           }

       }
       if { $name == "SortSas" } then {
          set status 1
          break;
       }
    }

    if { $status == 1 } then {
    #--> Resource present .....
       midas-check-for-temp
       set z [ catch { open "SortSas" w } f ] 
       if { $z == 0 } then {
          puts $f "((name $name)"
          puts $f " (host $host)"
          puts $f " (role $role)"
          puts $f " (type $type)"
          puts $f " (ipaddress 123456)"
          puts $f " (port $port))"
          close $f
        
         :apparatus free   :name "SortSas"
         :apparatus detach :name "SortSas"
       } else {
          midas-warning "Could not create SortSas"
       }
    } 

    #--> Now let us generate, claim and attach the correct resource
    midas-check-for-temp
    set z [ catch { open "SortSas" w } f ] 
    set name "SortSas"
    set host "localhost"
    set role "SortSas"
    set type "spectrum"
    set port $arg

    if { $z == 0 } then {
      puts $f "((name $name)"
      puts $f " (host $host)"
      puts $f " (role $role)"
      puts $f " (type $type)"
      puts $f " (ipaddress 123456)"
      puts $f " (port $port))"
      close $f
    
      setup_alloc $directory
    }
   
    return 4
}

proc setup_alloc arg {

    :apparatus attach :path $arg :name "SortSas"
    :apparatus claim  :name "SortSas"
}



