#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#-->                                                         <--#
#-->                                                         <--#
#--> D. Appelbe Daresbury NPG 06/30/03                       <--#
#-->  V Pucknell  2005
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#

proc get_settings {} {
    global midassortbase
    global directory
    global gaindirectory
    global gatedirectory
    global makefilepath
    global configtable
    global usegcc
    global sortdatadir
    global usesssnc
    global kmemsas


    :frame FSET :label "MIDASsort: Settings" \
    :icon-name "MIDASsort: Settings" :resize F :transient T

      :panel PSET1 :layout V :padx 5 :pady 5
       :layout H
       :message MSET :label "MIDASsort: Settings control frame" :bold T
       :next-row T
       :next-row T
       :text TSETMBDIR :label "MIDASsortBase" :value $midassortbase :inactive T :valuex 20c :w 55
       :next-row T
       :text TSETDIR :label "Sort directory" :value $directory :valuex 20c :w 55 :command set_directory
       :next-row T
       :text TSETDATADIR :label "Sort Data directory" :value $sortdatadir :valuex 20c :w 55 :command set_sortdatadir
       :next-row T
       :text TSETGAINDIR :label "Gainfile directory" :value $gaindirectory :valuex 20c :w 55 :command set_gaindirectory
       :next-row T
       :text TSETGATEDIR :label "Gatefile directory" :value $gatedirectory :valuex 20c :w 55 :command set_gatedirectory
       :next-row T
       :text TSETUM :label "Users Makefile" :value $makefilepath :valuex 20c :w 55 :command set_makefilepath
       :next-row T
       :text TSETTAB :label "Config Table" :value $configtable :valuex 20c :w 55 :command set_configtable
       :next-row T
       :checkbox CBUSEGCC :label "Use GNU compilers ?" :value $usegcc :valuex 30c :command set_usegcc
       :hskip 50
       :checkbox CBSSSNC :label "Sunsort spectrum name convention ?"  :value $usesssnc :valuex 30c :command set_usesssnc
       :next-row T
       :checkbox CBKMSAS :label "Kill memsas server on Exit ?" :value $kmemsas :valuex 30c :command set_kmemsas
       :hskip 50
       :button BMEMSASCTRL :label "Memsas Ctrl"    :command memsas_ctrl

       :next-row T
       :next-row T
       :button BSETREDISPLAY :label "Redisplay" :command update_settings
    
    :show T
}


proc update_settings {args} {
    global midassortbase
    global directory
    global gaindirectory
    global gatedirectory
    global makefilepath
    global usegcc
    global sortdatadir
    global usesssnc
    global configtable
    global kmemsas
    
    :modify TSETMBDIR     :value $midassortbase
    :modify TSETDIR       :value $directory
    :modify TSETDATADIR   :value $sortdatadir
    :modify TSETGAINDIR   :value $gaindirectory
    :modify TSETGATEDIR   :value $gatedirectory
    :modify TSETUM        :value $makefilepath
    :modify TSETTAB       :value $configtable
    :modify CBUSEGCC      :value $usegcc
    :modify CBSSSNC       :value $usesssnc
    :modify CBKMSAS       :value $kmemsas
}

#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> MIDAS interface for Sunsort                             <--#
#-->                                                         <--#
#--> save settings procedure                                 <--#
#-->                                                         <--#
#--> D. Appelbe Daresbury NPG 17/06/03                       <--#
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#

proc set_directory {w vl} {
    global directory

    set directory $vl
    SavePreferences
}

proc set_gaindirectory {w vl} {
    global gaindirectory

    set gaindirectory $vl
    SavePreferences
}

proc set_gatedirectory {w vl} {
    global gatedirectory

    set gatedirectory $vl
    SavePreferences
}

proc set_makefilepath {w vl} {
    global makefilepath

    set makefilepath $vl
    SavePreferences

}

proc set_configtable {w vl} {
    global configtable

    set configtable $vl
    SavePreferences

}

proc set_usegcc {w vl} {
    global usegcc

    set usegcc $vl
    SavePreferences

}

proc set_usesssnc {w vl} {
    global usesssnc

    set usesssnc $vl
    SavePreferences

}

proc set_kmemsas {w vl} {
    global kmemsas

    set kmemsas $vl
    SavePreferences
}

proc set_sortdatadir {w vl} {
    global sortdatadir

    set sortdatadir $vl
    SavePreferences
}

#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> preferences.tcl : This file contains all of the procs   <--#
#-->                   related to the use of the preferences <--#
#-->                   feature in MIDAS tcl                  <--#
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc LoadPreferences {} {
    global preferences
    global preflist
    global update_time input_type data_type
    global directory
    global gaindirectory
    global gatedirectory
    global env
    global makefilepath
    global configtable
    global usegcc
    global sortdatadir
    global usesssnc
    global kmemsas
    global diskffmt
    global uv1 uv2 uv3 uv4 uv5 uv6 uv7 uv8 uv9 uv10 uv11 uv12
    global uv13 uv14 uv15 uv16
    global vdpos vdaspos

    #--> Call the system routine to perform this load ...
    load_preferences

    #--> Now Are the preferences set .. if not set them to the default
    foreach item $preflist {
    switch $item {
        updateTime             { if {![info exists preferences($item)]} then { set preferences($item) $update_time }  }
        inputDevice            { if {![info exists preferences($item)]} then { set preferences($item) $input_type }  }
        inputFormat            { if {![info exists preferences($item)]} then { set preferences($item) $data_type }  }
        sortDir                { if {![info exists preferences($item)]} then { set preferences($item) $directory }  }
        sortDataDir            { if {![info exists preferences($item)]} then { set preferences($item) $sortdatadir }  }
        gainDir                { if {![info exists preferences($item)]} then { set preferences($item) $gaindirectory }  }
        gateDir                { if {![info exists preferences($item)]} then { set preferences($item) $gatedirectory }  }
        makeFilePath           { if {![info exists preferences($item)]} then { set preferences($item) $makefilepath }  }
        configTable            { if {![info exists preferences($item)]} then { set preferences($item) $configtable }  }
        useGCCcompilers        { if {![info exists preferences($item)]} then { set preferences($item) $usegcc }  }
        useSunSortSpecNameConv { if {![info exists preferences($item)]} then { set preferences($item) $usesssnc }  }
        killMemsasOnExit       { if {![info exists preferences($item)]} then {set preferences($item) $kmemsas } }
        inDiskFormat           { if {![info exists preferences($item)]} then {set preferences($item) $diskffmt } }
        userVar01              { if {![info exists preferences($item)]} then {set preferences($item) $uv1 } }
        userVar02              { if {![info exists preferences($item)]} then {set preferences($item) $uv2 } }
        userVar03              { if {![info exists preferences($item)]} then {set preferences($item) $uv3 } }
        userVar04              { if {![info exists preferences($item)]} then {set preferences($item) $uv4 } }
        userVar05              { if {![info exists preferences($item)]} then {set preferences($item) $uv5 } }
        userVar06              { if {![info exists preferences($item)]} then {set preferences($item) $uv6 } }
        userVar07              { if {![info exists preferences($item)]} then {set preferences($item) $uv7 } }
        userVar08              { if {![info exists preferences($item)]} then {set preferences($item) $uv8 } }
        userVar09              { if {![info exists preferences($item)]} then {set preferences($item) $uv9 } }
        userVar10              { if {![info exists preferences($item)]} then {set preferences($item) $uv10 } }
        userVar11              { if {![info exists preferences($item)]} then {set preferences($item) $uv11 } }
        userVar12              { if {![info exists preferences($item)]} then {set preferences($item) $uv12 } }
        userVar13              { if {![info exists preferences($item)]} then {set preferences($item) $uv13 } }
        userVar14              { if {![info exists preferences($item)]} then {set preferences($item) $uv14 } }
        userVar15              { if {![info exists preferences($item)]} then {set preferences($item) $uv15 } }
        userVar16              { if {![info exists preferences($item)]} then {set preferences($item) $uv16 } }
        vdposGLOB              { if {![info exists preferences($item)]} then {set preferences($item) $vdpos } }
        vdasposGLOB            { if {![info exists preferences($item)]} then {set preferences($item) $vdaspos } }
    }
    }

    #--> Now set all of our variables to what is in the pref list
    foreach item $preflist {
    switch $item {
        updateTime             { set update_time $preferences($item) }
        inputDevice            { set input_type $preferences($item) }
        inputFormat            { set data_type $preferences($item) }
        sortDir                { set directory $preferences($item) }
        sortDataDir            { set sortdatadir $preferences($item) }
        gainDir                { set gaindirectory $preferences($item) }
        gateDir                { set gatedirectory $preferences($item) }
        makeFilePath           { set makefilepath $preferences($item) }
        configTable            { set configtable $preferences($item) }
        useGCCcompilers        { set usegcc $preferences($item) }
        useSunSortSpecNameConv { set usesssnc $preferences($item) }
        killMemsasOnExit       { set kmemsas $preferences($item) }
        inDiskFormat           { set diskffmt $preferences($item) }
        userVar01              { set uv1 $preferences($item) }
        userVar02              { set uv2 $preferences($item) }
        userVar03              { set uv3 $preferences($item) }
        userVar04              { set uv4 $preferences($item) }
        userVar05              { set uv5 $preferences($item) }
        userVar06              { set uv6 $preferences($item) }
        userVar07              { set uv7 $preferences($item) }
        userVar08              { set uv8 $preferences($item) }
        userVar09              { set uv9 $preferences($item) }
        userVar10              { set uv10 $preferences($item) }
        userVar11              { set uv11 $preferences($item) }
        userVar12              { set uv12 $preferences($item) }
        userVar13              { set uv13 $preferences($item) }
        userVar14              { set uv14 $preferences($item) }
        userVar15              { set uv15 $preferences($item) }
        userVar16              { set uv16 $preferences($item) }
        vdposGLOB              { set vdpos $preferences($item) }
        vdasposGLOB            { set vdaspos $preferences($item) }
    }
    }

    SetEnvironment

}

proc SavePreferences {} {
    global preferences
    global preflist
    global update_time input_type data_type
    global directory
    global gaindirectory
    global gatedirectory
    global env
    global makefilepath
    global configtable
    global usegcc
    global sortdatadir
    global usesssnc
    global kmemsas
    global diskffmt
    global uv1 uv2 uv3 uv4 uv5 uv6 uv7 uv8 uv9 uv10 uv11 uv12
    global uv13 uv14 uv15 uv16
    global vdpos vdaspos

    #--> Now set all of our variables to what is in the pref list
    foreach item $preflist {
    switch $item {
        updateTime             { set preferences($item) $update_time }
        inputDevice            { set preferences($item) $input_type }
        inputFormat            { set preferences($item) $data_type }
        sortDir                { set preferences($item) $directory }
        sortDataDir            { set preferences($item) $sortdatadir }
        gainDir                { set preferences($item) $gaindirectory }
        gateDir                { set preferences($item) $gatedirectory }
        makeFilePath           { set preferences($item) $makefilepath }
        configTable            { set preferences($item) $configtable }
        useGCCcompilers        { set preferences($item) $usegcc }
        useSunSortSpecNameConv { set preferences($item) $usesssnc }
        killMemsasOnExit       { set preferences($item) $kmemsas }
        inDiskFormat           { set preferences($item) $diskffmt }
        userVar01              { set preferences($item) $uv1 }
        userVar02              { set preferences($item) $uv2 }
        userVar03              { set preferences($item) $uv3 }
        userVar04              { set preferences($item) $uv4 }
        userVar05              { set preferences($item) $uv5 }
        userVar06              { set preferences($item) $uv6 }
        userVar07              { set preferences($item) $uv7 }
        userVar08              { set preferences($item) $uv8 }
        userVar09              { set preferences($item) $uv9 }
        userVar10              { set preferences($item) $uv10 }
        userVar11              { set preferences($item) $uv11 }
        userVar12              { set preferences($item) $uv12 }
        userVar13              { set preferences($item) $uv13 }
        userVar14              { set preferences($item) $uv14 }
        userVar15              { set preferences($item) $uv15 }
        userVar16              { set preferences($item) $uv16 }
        vdposGLOB              { set preferences($item) $vdpos }
        vdasposGLOB            { set preferences($item) $vdaspos }
    }
    }
    #--> Now Save things ....
    update_preferences

    SetEnvironment

}


proc SetEnvironment {} {
    global env usegcc


    switch $env(os)  {
        SunOS          {
             if {$usegcc == 0} {
                 set env(PATH) [file join $env(MIDASsortBase) SunOS bin_cc]:$env(PATH)
             } else {
                 set env(PATH) [file join $env(MIDASsortBase) SunOS bin_gcc]:$env(PATH)
             }
        }
        Linux          {set env(PATH) [file join $env(MIDASsortBase) Linux bin]:$env(PATH)}
        Win            {
             if {$usegcc == 0} {
                 set env(PATH) [file join $env(MIDASsortBase) Win bin]\;[file join $env(MIDASBASE) bin_Win]\;$env(PATH)
             } else {
                 set env(PATH) [file join $env(MIDASsortBase) Cygwin bin]\;[file join $env(MIDASsortBase) Win bin]\;[file join $env(MIDASBASE) bin_Win]\;$env(PATH)
             }
        }
        default {}
    }

    if {[info exists env(LD_LIBRARY_PATH)]} {
        switch $env(os)  {
            SunOS          {
                if {$usegcc == 0} {
                   set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) SunOS lib_cc]:$env(LD_LIBRARY_PATH)
                } else {
                   set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) SunOS lib_gcc]:$env(LD_LIBRARY_PATH)
                }
            }
            Linux          {set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) Linux lib]:$env(LD_LIBRARY_PATH)}
            Win            {
                if {$usegcc == 0} {
                   set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) Win bin]\;[file join $env(MIDASBASE) bin_Win]\;$env(LD_LIBRARY_PATH)
                } else {
                   set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) Cygwin bin]\;[file join $env(MIDASsortBase) Win bin]\;[file join $env(MIDASBASE) bin_Win]\;$env(LD_LIBRARY_PATH)
                }
            }
            default {}
        }
    } else {
        switch $env(os)  {
            SunOS          {
                if {$usegcc == 0} {
                   set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) SunOS lib_cc]
                } else {
                   set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) SunOS lib_gcc]
                }
            }
            Linux          {set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) Linux lib]}
            Win            {
                if {$usegcc == 0} {
                   set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) Win bin]\;[file join $env(MIDASBASE) bin_Win]
                } else {
                   set env(LD_LIBRARY_PATH) [file join $env(MIDASsortBase) Cygwin bin]\;[file join $env(MIDASsortBase) Win bin]\;[file join $env(MIDASBASE) bin_Win]
                }
            }
           default {}
       }
    }

    switch $env(os)  {
         SunOS          {
             if {$usegcc == 0} {
                 set env(MSPLUGINDIR) [file join $env(MIDASsortBase) SunOS lib_cc]
             } else {
                 set env(MSPLUGINDIR) [file join $env(MIDASsortBase) SunOS lib_gcc]
             }
         }
         Linux          {set env(MSPLUGINDIR) [file join $env(MIDASsortBase) Linux lib]}
         Win            {
             if {$usegcc == 0} {
                 set env(MSPLUGINDIR) [file join $env(MIDASsortBase) Win bin]
             } else {
                 set env(MSPLUGINDIR) [file join $env(MIDASsortBase) Cygwin bin]
             }
         }
         default {}
    }

}
